/*
	File:		Window.h

	Contains:	Definition of TWindow, a base class which provides a
				framework for building way-cool windows which even John
				Sullivan would be happy with. Floating windows and smart
				zooming algorithms are based on code samples provided by
				Dean Yu.
				
	Written by: Dave Falkenburg, Dean Yu

	Copyright:	 1993-1995 by Dave Falkenburg, all rights reserved.

	Change History (most recent first):
	 
		 <8>	 1/24/95	DRF		Make DoMenuSelection return a Boolean, just like DoMenuCommand.
		 <7>	 1/20/95	DRF		Add a field to track whether a TWindow needs to be disposed via
									DisposeDialog. Thanks to Gary W. Powell @ Adobe for reporting
									the bug.
		 <6>	  1/3/95	DRF		Add Nitins changes for Drag handling: a ClickAndDrag method.
		 <5>	  1/3/95	DRF		Protected TWindow::Window because it is not legal to create a
									TWindow without subclassing. Also revised DoMenuSelection to use
									a typedef-ed parameter.
		 <4>	11/12/94	DRF		Added AdjustMenusBeforeMenuSelection.
		 <3>	 11/8/94	DRF		Add some better menu handling methods.
		 <2>	  9/4/94	DRF		Added DrawJustTheGrowIcon.
 */

#ifndef		_WINDOW_
#define		_WINDOW_

#ifndef		__TYPES__
#include	<Types.h>
#endif

#ifndef		__WINDOWS__
#include	<Windows.h>
#endif

#ifndef		__EVENTS__
#include	<Events.h>
#endif

#ifndef		__DRAG__
#include	<Drag.h>
#endif

#ifndef		_MENUBAR_
#include	"MenuBar.h"
#endif


typedef	short	WindowTemplateID;


class	TWindow
	{
protected:
	//	protect the constructor because it isnt legal to create a TWindow directly
	
						TWindow();
public:

	virtual				~TWindow();

	//	Event routing methods

	virtual	Boolean		EventFilter(EventRecord * theEvent);	

	//	Methods you shouldnt need to override, but might need to

	enum WindowType
		{
		kNormalWindow = 0,
		kFloatingWindow,
		kModalWindow
		};

	virtual void		CreateWindow(WindowType typeOfWindowToCreate = kNormalWindow);
	virtual	void		Select(void);
	virtual	void		Drag(Point startPoint);
	virtual void		Nudge(short horizontalDistance, short verticalDistance);
	virtual void		Grow(Point startPoint);
	virtual void		Zoom(short zoomState);

	virtual	void		ShowHide(Boolean showFlag);


	//	Methods which MUST be overridden:

	virtual WindowPtr	MakeNewWindow(WindowPtr behindWindow)	= 0;
	

	//	Methods which probably should be overridden

	virtual	void		AdjustCursor(EventRecord * anEvent);
	virtual void		Idle(EventRecord * anEvent);
	virtual void		Activate(Boolean activating);
	virtual void		Draw(void);
	virtual void		Click(EventRecord * anEvent);
	virtual	void		KeyDown(EventRecord * anEvent);

	virtual void		GetPerfectWindowSize(Rect * perfectSize);
	virtual	void		GetWindowSizeLimits(Rect * limits);
	virtual void		AdjustForNewWindowSize(Rect * oldRect,Rect * newRect);


	//	Window property accessor methods
	//		watch for new ones when we add scripting support	
	
	virtual	Boolean		IsVisible(void);
	
	virtual	Boolean		CanClose(void);		
	virtual	Boolean		Close(void);
	virtual	Boolean		DeleteAfterClose(void);
	
	//	Methods for handling menus & menu commands
	
	virtual	void		AdjustMenusBeforeMenuSelection(void);
	virtual Boolean		DoMenuSelection(short menu, short item);
	virtual Boolean		DoMenuCommand(MenuCommandID menuCommand);

	//	Methods for use with the Drag Manager
	
	virtual	void		ClickAndDrag(EventRecord * anEvent);
						//	NOTE:	This method takes GLOBAL coordinates
						//			because thats what TrackDrag expects.
	
	virtual	OSErr		HandleDrag(DragTrackingMessage dragMessage,DragReference theDrag);

	virtual	OSErr		DragEnterWindow(DragReference theDrag);
	virtual	OSErr		DragInWindow(DragReference theDrag);
	virtual	OSErr		DragLeaveWindow(DragReference theDrag);

	virtual	OSErr		HandleDrop(DragReference theDragRef);


protected:
	WindowPtr			fWindow;
	WindowType			fWindowType;

	Boolean				fIsDialogWindow;
	Boolean				fIsVisible;
	};


//	Utility Functions:

//	Dont you just wish you didnt have to do this?
pascal WindowPtr	GetNewColorOrBlackAndWhiteWindow(short windowID, void *wStorage, WindowPtr behind);
pascal WindowPtr	NewColorOrBlackAndWhiteWindow(void *wStorage, const Rect *boundsRect, ConstStr255Param title, Boolean visible, short theProc, WindowPtr behind, Boolean goAwayFlag, long refCon);

void				DrawJustTheGrowIcon(WindowPtr theWindow);

TWindow *			GetWindowObject(WindowPtr aWindow);

WindowPtr			FrontModalWindow(void);
WindowPtr			LastModalWindow(void);
WindowPtr			FrontFloatingWindow(void);
WindowPtr			LastFloatingWindow(void);
WindowPtr			FrontNonFloatingWindow(void);

void				HiliteAndActivateWindow(WindowPtr aWindow,Boolean active);
void				SuspendResumeWindows(Boolean resuming);
void				HiliteWindowsForModalDialog(Boolean hiliting);

pascal OSErr		CallWindowDragTrackingHandler(DragTrackingMessage message,WindowPtr theWindow,void *handlerRefCon,DragReference theDragRef);
pascal OSErr		CallWindowDragReceiveHandler(WindowPtr theWindow, void *handlerRefCon,DragReference theDragRef);

#endif
